# **Mobroute:** CLI Userguide

[Mobroute](http://sr.ht/~mil/mobroute) is a general purpose FOSS public
transportation router (e.g. trip planner) Go library and CLI that works by
directly ingesting timetable (GTFS) data from transit agencies themselves
(sourced from the Mobility Database). For an overview of Mobroute's
functionality see [Mobroute's project page](http://sr.ht/~mil/mobroute).
This document is a guide on how to run Mobroute's CLI by compiling the
Go codebase and then running routing requests locally.

- *1:* [Primer Documentation](#strongprimer-documentationstrong)
- *2:* [Mobroute CLI: Install](#strongmobroute-cli-installstrong)
- *3:* [Mobroute CLI: Usage](#strongmobroute-cli-usagestrong)
- *4:* [Mobroute CLI: Route subcommand](#strongmobroute-cli-route-subcommandstrong)
- *5:* [Mobroute CLI: Mobsql subcommand](#strongmobroute-cli-mobsql-subcommandstrong)
- *6:* [Further Resources & Documentation](#strongfurther-resources-amp-documentationstrong)

## **Primer Documentation**

- Before using Mobroute's CLI, please read the following documentation:
  - [Mobroute: Project Overview & Featureset](https://git.sr.ht/~mil/mobroute)
  - [Mobroute: Finding GTFS Feeds](https://git.sr.ht/~mil/mobroute/tree/master/doc/doc_gtfsfeeds.md)
  - [Mobroute: Algorithm Tunable Parameters](https://git.sr.ht/~mil/mobroute/tree/master/doc/doc_tunables.md)

## **Mobroute CLI: Install**

- Install via package:
  - On some distrobutions the Mobroute CLI has been packaged, if available
    this is the simplest installation method.
  - Distrobution packages:
    - Alpine Linux Testing: `apk add mobroute`


- Install from source:
  - If a package is not avilabale for your distrobution, building from source
    is simple using Go
  - Steps:
    - (1) Installation assumes Go (>=1.18)
    - (2) Clone the Mobroute repository:
      - `git clone http://git.sr.ht/~mil/mobroute`
    - (3) Build the `mobroute` binary:
      - `./build.sh`
    - (4) Install as:
      - `cp mobroute /usr/bin/`

## **Mobroute CLI: Usage**
```sh
Usage of ./mobroute:
  mobroute [subcommand]

Description:
  Mobroute is a general purpose FOSS public transportation router (e.g. trip planner). See: http://sr.ht/~mil/mobroute

Subcommands:
  - route: Perform a routing request from GTFS dataload, to CSA algorithm calculation, and formatting
  - mobsql: Diagnostic tool which can query and manipulate the underlying SQLiteDB via the underlying mobsql library
  - version: Display the version information for this release
```

## **Mobroute CLI: Route subcommand**

```sh
Usage of ./mobroute route:

Description:
  The route subcommand is the primary command for the CLI (and is a passthrough to the underlying Mobroute library OneshotRoute function). This subcommand allows you to perform routing calculations using the CSA algorithm. All functionality from loading GTFS data (by feed id), to loading the required data to memory, to performing the routing calculation via the CSA algorithm, and finally to formatting the result in several formats is handled by this single CLI subcommand.

Flags:
  -d string
        Verbose debug categories: (i) Information (w) Warn (d) Debug (default "i")
  -dprof string
        Write debug pprof CPU profile to file
  -h    Show this usage message
  -rp string

        JSON Parameters for oneshot route subcommand, should be in format of: {"route_params": {"feed_ids": [516],...}}

        Required route parameters:
          - feed_ids: Array of integer feed IDs (MDBID) to use for routing requests (ex: [516, 510])
          - from: Array of two floats in lat, lon format (ex: [50.012338, 19.88192])
          - to: Array of two floats in lat, lon format (ex: [50.012338, 19.88192])
          - transfer_categories: Array of transfers categories to load - may include "f" for feed, "i" for implicit, "g" for generated (ex: ["f", "i"])
          - output_formats: Array of formats for the output may include - may include "legs", "geojson", "diagnostics", "mapurl", "request" (ex: ["legs", "geojson"])

        Optional route parameters:
          - time: Datetime string at which route should depart in RFC3339 format (default: current system time)
          - max_walk_seconds: Maximum seconds walking distance on start/end of trip to get to/from origin/destination stops (default: 1200)
          - max_trip_seconds: Maximum seconds the entire route/trip can take (default: 14400)
          - min_transfer_seconds: Minimum seconds each transfer in the route can take (default: 180)
          - max_transfer_seconds: Maximum seconds each transfer in the route can take (default: 2400)
          - max_n_transfers: Maximum number of transfers a route can contain (default: 20)
          - walkspeed_km_hr: Ratio of kilometers per hour the user walk (default: 4.5)


Examples:
  NYC Routing Example:
    mobroute route -rp '{"route_params":{"feed_ids":[516],"from":[40.70940,-74.00537],"to":[40.72879,-73.95215],"transfer_categories":["f","i"],"output_formats":["legs","diagnostics","mapurl","request"]}}'
  Brussels Routing Example:
    mobroute route -rp '{"route_params":{"feed_ids":[1088],"from":[50.85728,4.351426],"to":[50.83214,4.350534],"transfer_categories":["f","i"],"output_formats":["legs","diagnostics","mapurl","request"]}}'
  Krakow Routing Example:
    mobroute route -rp '{"route_params":{"feed_ids":[1270],"from":[50.012338,19.88192],"to":[50.08785,20.02387],"transfer_categories":["f","i"],"output_formats":["legs","diagnostics","mapurl","request"]}}'
```


## **Mobroute CLI: Mobsql subcommand**

```sh
Usage of ./mobroute mobsql:

Description:
  The mobsql subcommand is a diagnostic command (and a passthrough to the underlying Mobroute library OneshotMobsql function) which allows you to both alter and query the status of GTFS (and GTFS-derived data) in the local SQLite database.
 In normal scenarios & functioning end-users shouldn't need to use this subcommand at all as this is just a debugging tool. See mobsql documentation for specifics on the functionality of each operation (op).

Flags:
  -d string
        Verbose debug categories: (i) Information (w) Warn (d) Debug (default "i")
  -dprof string
        Write debug pprof CPU profile to file
  -h    Show this usage message
  -mp string

        JSON Parameters for oneshot mobsql subcommand, should be in format of: {"mobsql_params": {"feed_ids": [767], "op": "status"}}

        Required mobsql parameters:
          - feed_ids: Array of integer feed IDs (MDBID) to use for mobsql requests (ex: [767])
          - op: Operation to perform - may be one of "status", "load", "compute", "purge_gtfs", "purge_computed", "purge_all" (ex: "status")


Examples:
  Determine GTFS Feed Status:
    mobroute mobsql -mp '{"mobsql_params": {"op": "status", "feed_ids": [767]}}'
  Load a GTFS Feed:
    mobroute mobsql -mp '{"mobsql_params": {"op": "load", "feed_ids": [767]}}'
  Compute Routing-Optimized GTFS Derived Tables:
    mobroute mobsql -mp '{"mobsql_params": {"op": "compute", "feed_ids": [767]}}'
  Purge GTFS Feed Data:
    mobroute mobsql -mp '{"mobsql_params": {"op": "purge_gtfs", "feed_ids": [767]}}'
  Purge Computed Tables:
    mobroute mobsql -mp '{"mobsql_params": {"op": "purge_computed", "feed_ids": [767]}}'
  Purge All Tables:
    mobroute mobsql -mp '{"mobsql_params": {"op": "purge_all", "feed_ids": [767]}}'
```

## **Further Resources & Documentation**

- [Automated Test Results / Metros](https://ci.lrdu.org/tests_results/): If you're curious about metros that have been well-tested and MDBIDs which you can experiment with for routing, this page has a number of working routing requests (lat/lon pairs and MDBIDs)
- [Development Guide](doc_dev.md): If you're curious about how Mobroute works under-the-hood, this guide can help architecture overview of what happens when you submit a routing request, from ETL, to algorithm, to formatting
- [Contribute Guide](doc_contribute.md): If you're intrested in contributing, this goes over the workflow.
- [Transito](http://git.sr.ht/~mil/transito): The Transito Mobile app runs the same routing logic the CLI executes on a mobile app (Android & Linux)

---

> [All Mobroute Docs](http://sr.ht/~mil/mobroute)
